@@ -1,9 +1,10 @@
-/* 29oct11abu
+/* 02dec11abu
  * (c) Software Lab. Alexander Burger
  */
 
 var FormReq = false;
 var HintReq = false;
+var Hint, Hints, Item, Beg, End;
 
 try {
    FormReq = new XMLHttpRequest();
@@ -21,6 +22,14 @@ function inBtn(flg) {InBtn = flg;}
 
 function formKey(event) {
    Key = event.keyCode;
+   if (Hint  &&  Hint.style.visibility == "visible") {
+      if ((Item >= 0 && Key == 13) || Key == 38 || Key == 40)
+         return false;
+      if (event.keyCode == 27) {
+         Hint.style.visibility = "hidden";
+         return false;
+      }
+   }
    return true;
 }
 
@@ -59,6 +68,13 @@ function dropLoad(event) {
    Drop = null;
 }
 
+function hasElement(form, name) {
+   for (var i = 0; i < form.elements.length; ++i)
+      if (form.elements[i].name == name)
+         return true;
+   return false;
+}
+
 /*** Form submit ***/
 function doPost(form) {
    for (var i = 0; ; ++i) {
@@ -71,21 +87,21 @@ function doPost(form) {
 }
 
 function post(form, file) {
-   var i, j, url, data;
+   var i;
 
-   if (!FormReq)
+   if (!FormReq || !hasElement(form,"*Get") || (i = form.action.indexOf("~")) <= 0)
       return true;
    if (FormReq.readyState > 0 && FormReq.readyState < 4) {
       Queue.push(form);
       return false;
    }
    form.style.cursor = "wait";
-   url = form.action.split("~");
-   try {FormReq.open("POST", url[0] + "~!jsForm?" + url[1]);}
+   try {FormReq.open("POST", form.action.substr(0,i) + "~!jsForm?" + form.action.substr(i+1));}
    catch (e) {return true;}
-
    FormReq.onreadystatechange = function() {
       if (FormReq.readyState == 4 && FormReq.status == 200) {
+         var i, j;
+
          if (FormReq.responseText == "T") {
             Queue.length = 0;
             form.submit();
@@ -225,9 +241,8 @@ function post(form, file) {
             post(Queue.shift(), null);
       }
    }
-
-   data = "";
-   for (i = 0; i < Btn.length;)
+   var data = "";
+   for (var i = 0; i < Btn.length;)
       if (Btn[i].form != form)
          ++i;
       else {
@@ -274,109 +289,174 @@ function post(form, file) {
    return false;
 }
 
-
 /*** Hints ***/
-var Hint, Pos;
-
 function doHint(field) {
-   var i, url, data;
+   if (HintReq) {
+      if (!Hint) {
+         Hint = document.createElement("div");
+         Hint.setAttribute("class", "hint");
+         Hint.style.visibility = "hidden";
+         Hint.style.position = "absolute";
+         Hints = document.createElement("div");
+         Hints.setAttribute("class", "hints");
+         Hints.style.position = "relative";
+         Hints.style.top = "-2px";
+         Hints.style.left = "-3px";
+         Hint.appendChild(Hints);
+      }
+      field.parentNode.appendChild(Hint);
+      field.onblur = function() {
+         Hint.style.visibility = "hidden";
+      }
+      var top = field.offsetHeight + 2;
+      var left = 3;
+      for (var obj = field;  obj.id != "main" && obj.id != "menu";  obj = obj.offsetParent) {
+         top += obj.offsetTop;
+         left += obj.offsetLeft;
+      }
+      Hint.style.top = top + "px";
+      Hint.style.left = left + "px";
+   }
+}
+
+function hintKey(field, event, tok, coy) {
+   var i, data;
 
-   Hint = null;
    if (!HintReq)
       return true;
+   if (event.keyCode == 9 || event.keyCode == 27)
+      return false;
+   if (Hint.style.visibility == "visible") {
+      if (Item >= 0 && event.keyCode == 13) {
+         setHint(field, Hints.childNodes[Item]);
+         return false;
+      }
+      if (event.keyCode == 38) {  // Up
+         if (Item > 0) {
+            hintOff(Item);
+            hintOn(--Item);
+         }
+         return false;
+      }
+      if (event.keyCode == 40) {  // Down
+         if (Item < (lst = Hints.childNodes).length-1) {
+            if (Item >= 0)
+               hintOff(Item);
+            hintOn(++Item);
+         }
+         return false;
+      }
+   }
+   if (event.keyCode == 13)
+      return true;
    if (HintReq.readyState > 0 && HintReq.readyState < 4)
       return false;
-   if ((i = field.id.lastIndexOf("-")) < 0)
-      return true;
-   url = field.form.action.split("~");
-   try {HintReq.open("POST", url[0] + "~!jsHint?" + field.id.substr(i+1));}
+   if (tok) {
+      for (Beg = field.selectionStart;  Beg > 0 && !field.value.charAt(Beg-1).match(/\s/);  --Beg);
+      End = field.selectionEnd;
+   }
+   else {
+      Beg = 0;
+      End = field.value.length;
+   }
+   if (event.keyCode != 45) {  // INS
+      if (Beg == End) {
+         Hint.style.visibility = "hidden";
+         return false;
+      }
+      if (coy && Hint.style.visibility == "hidden")
+         return false;
+   }
+   try {
+      HintReq.open("POST",
+         ((i = field.form.action.indexOf("~")) <= 0?
+            "" : field.form.action.substr(0, i+1) ) +
+         ((i = field.id.lastIndexOf("-")) < 0?
+            "!jsHint?$" + field.id : "!jsHint?+" + field.id.substr(i+1) ) );
+   }
    catch (e) {return true;}
    HintReq.onreadystatechange = function() {
       if (HintReq.readyState == 4 && HintReq.status == 200) {
-         Hint = HintReq.responseText.split("&");
-         for (i = 0; i < Hint.length; ++i)
-            Hint[i] = decodeURIComponent(Hint[i]);
+         var i, n, lst, str;
+
+         if ((str = HintReq.responseText).length == 0)
+            Hint.style.visibility = "hidden";
+         else {
+            lst = str.split("&");
+            while (Hints.hasChildNodes())
+               Hints.removeChild(Hints.firstChild);
+            for (i = 0, n = 7; i < lst.length; ++i) {
+               addHint(i, field, str = decodeURIComponent(lst[i]));
+               if (str.length > n)
+                  n = str.length;
+            }
+            Hints.style.width = n + 3 + "ex";
+            Hint.style.width = n + 4 + "ex";
+            Hint.style.visibility = "visible";
+            Item = -1;
+         }
       }
    }
+   var data = "*JsHint=" + encodeURIComponent(field.value.substring(Beg,End));
    for (i = 0; i < field.form.elements.length; ++i) {
       var fld = field.form.elements[i];
 
       if (fld.name == "*Get")
-         data = "*Get=" + fld.value;
+         data += "&*Get=" + fld.value;
       else if (fld.name == "*Form")
          data += "&*Form=" + fld.value;
    }
    try {HintReq.send(data);}
    catch (e) {HintReq.abort();}
-   Pos = -1;
-   return true;
+   return (event.keyCode != 45);  // INS
 }
 
-function hintKey(field, event, coy) {
-   var beg = field.selectionStart;
-   var end = field.selectionEnd;
-   var i;
-
-   if (Hint.length > 0) {
-      if (event.keyCode == 19) {  // Pause/Break
-         if (beg != end)
-            return true;
-         for (Pos = beg;  Pos > 0 && !field.value.charAt(Pos-1).match(/\s/);  --Pos);
-         if ((i = findHint(field.value.substring(Pos, beg))) < 0)
-            Pos = -1;
-         else
-            setHint(field, beg, end, i);
-         return false;
-      }
-      if (event.keyCode == 38 || event.keyCode == 40) {  // Up or Down
-         if (beg == end)
-            return true;
-         if ((i = findHint(field.value.substring(Pos, end))) >= 0)
-            setHint(field, beg, end, nextHint(field.value.substring(Pos, beg), i, event.keyCode==38? -1 : +1));
-         return false;
-      }
-      if (!coy) {
-         if (Pos < 0)
-            for (Pos = beg;  Pos > 0 && !field.value.charAt(Pos-1).match(/\s/);  --Pos);
-         if ((i = findHint(field.value.substring(Pos, beg) + String.fromCharCode(event.charCode || event.keyCode))) < 0)
-            Pos = -1;
-         else {
-            setHint(field, beg+1, end, i);
-            return false;
-         }
-      }
+function addHint(i, field, str) {
+   var item = document.createElement("div");
+   item.appendChild(document.createTextNode(str));
+   item.onmouseover = function() {
+      if (Item >= 0)
+         hintOff(Item);
+      hintOn(i);
+      field.onblur = false;
+      field.onchange = false;
+      Item = i;
    }
-   return true;
-}
-
-function findHint(str) {
-   var len = str.length;
-
-   for (var i = 0; i < Hint.length; ++i)
-      if (Hint[i].substr(0,len) == str)
-         return i;
-   return -1;
-}
-
-function nextHint(str, i, n) {
-   var len = str.length;
-
-   do {
-      if (n < 0) {
-         if ((i += n) < 0)
-            i = Hint.length - 1;
+   item.onmouseout = function() {
+      hintOff(Item);
+      field.onblur = function() {
+         Hint.style.visibility = "hidden";
       }
-      else {
-         if ((i += n) >= Hint.length)
-               i = 0;
-      }
-   } while (Hint[i].substr(0,len) != str);
-   return i;
+      field.onchange = function() {
+         return fldChg(field, item);
+      };
+      Item = -1;
+   }
+   item.onclick = function() {
+      setHint(field, item);
+   }
+   Hints.appendChild(item);
 }
 
-function setHint(field, beg, end, i) {
-   field.value = field.value.substr(0,Pos) + Hint[i] + field.value.substring(end, field.value.length);
-   field.setSelectionRange(beg, Pos+Hint[i].length);
-   field.onblur = function() {fldChg(field)};
-   field.onchange = false;
+function setHint(field, item) {
+   Hint.style.visibility = "hidden";
+   field.value = field.value.substr(0,Beg) + item.firstChild.nodeValue + field.value.substr(End);
+   post(field.form, null);
+   field.setSelectionRange(Beg + item.firstChild.nodeValue.length, field.value.length);
+   field.focus();
+   field.onchange = function() {
+      return fldChg(field)
+   };
+}
+
+function hintOn(i) {
+   var s = Hints.childNodes[i].style;
+   s.background = "black";
+   s.color= "white";
+}
+
+function hintOff(i) {
+   var s = Hints.childNodes[i].style;
+   s.background = "white";
+   s.color= "black";
 }
