@@ -1,4 +1,4 @@
-/* 10jun11abu
+/* 27oct11abu
  * (c) Software Lab. Alexander Burger
  */
 
@@ -28,7 +28,7 @@ any Alarm, Sigio, Line, Zero, One;
 any Intern[IHASH], Transient[IHASH], Extern[EHASH];
 any ApplyArgs, ApplyBody, DbVal, DbTail;
 any Nil, DB, Meth, Quote, T;
-any Solo, PPid, Pid, At, At2, At3, This, Dbg, Zap, Ext, Scl, Class;
+any Solo, PPid, Pid, At, At2, At3, This, Prompt, Dbg, Zap, Ext, Scl, Class;
 any Run, Hup, Sig1, Sig2, Up, Err, Msg, Uni, Led, Tsm, Adr, Fork, Bye;
 bool Break;
 sig_atomic_t Signal[NSIG];
@@ -302,7 +302,9 @@ any doEnv(any x) {
          if (isCell(data(c2))) {
             do
                data(c1) = cons(
-                  isCell(car(data(c2)))? car(data(c2)) : cons(car(data(c2)), val(car(data(c2)))),
+                  isCell(car(data(c2)))?
+                     cons(caar(data(c2)), cdar(data(c2))) :
+                     cons(car(data(c2)), val(car(data(c2)))),
                   data(c1) );
             while (isCell(data(c2) = cdr(data(c2))));
          }
@@ -660,20 +662,20 @@ any evExpr(any expr, any x) {
       ++f.cnt, x = cdr(x), y = cdr(y);
    }
    if (isNil(y)) {
-      while (--f.i > 0) {
-         x = val(f.bnd[f.i].sym);
+      do {
+         x = val(f.bnd[--f.i].sym);
          val(f.bnd[f.i].sym) = f.bnd[f.i].val;
          f.bnd[f.i].val = x;
-      }
+      } while (f.i);
       x = prog(cdr(expr));
    }
    else if (y != At) {
       f.bnd[f.cnt].sym = y,  f.bnd[f.cnt++].val = val(y),  val(y) = x;
-      while (--f.i > 0) {
-         x = val(f.bnd[f.i].sym);
+      do {
+         x = val(f.bnd[--f.i].sym);
          val(f.bnd[f.i].sym) = f.bnd[f.i].val;
          f.bnd[f.i].val = x;
-      }
+      } while (f.i);
       x = prog(cdr(expr));
    }
    else {
@@ -683,11 +685,11 @@ any evExpr(any expr, any x) {
 
       while (--n >= 0)
          Push(c[n], EVAL(car(x))),  x = cdr(x);
-      while (--f.i > 0) {
-         x = val(f.bnd[f.i].sym);
+      do {
+         x = val(f.bnd[--f.i].sym);
          val(f.bnd[f.i].sym) = f.bnd[f.i].val;
          f.bnd[f.i].val = x;
-      }
+      } while (f.i);
       n = Env.next,  Env.next = cnt;
       arg = Env.arg,  Env.arg = c;
       x = prog(cdr(expr));
